/**
 * Introduction
 *
 * @title ROV - Easy applications from C
 *
 * @copyright Jason Tribbeck / 7th ARM 1994-2001
 *
 * @authors Jason Tribbeck
 *
 * @file draw.h
 *
 * @type subsystem
 *
 * @description
 *
 */

#ifndef __rov_draw
#define __rov_draw


#ifdef __cplusplus
extern "C"
{
#endif

#ifndef __rov_main
#include "main.h"
#endif

#ifndef __rov_canvas
#include "canvas.h"
#endif

#ifndef __rov_icons
#include "icons.h"
#endif

/**
 * Structure for a sprite translation table.
 */
typedef struct {
  int xm;
  int ym;
  int xd;
  int yd;
  int xt;
  int yt;
} spr_transtab;

/**
 * Interface to OS_Plot, but uses canvas' origin instead of
 * screen origin.
 * @param code the plot code.
 * @param x the (canvas) x coordinate.
 * @param y the (canvas) y coordinate.
 */
void draw_plot(int code,
	       int x,
	       int y);

/**
 * Perform a MOVE to canvas location
 * @mparam x the (canvas) x coordinate.
 * @mparam y the (canvas) y coordinate.
 */
#define	draw_move(x,y) draw_plot(4,x,y)

/**
 * Perform a DRAW to canvas location.
 * @mparam x the (canvas) x coordinate.
 * @mparam y the (canvas) y coordinate.
 */
#define	draw_line(x,y) draw_plot(5,x,y)

/**
 * Perform a relative MOVE
 * @mparam x the x offset.
 * @mparam y the y offset.
 */
#define	draw_moveto(x,y) draw_plot(0,x,y)

/**
 * Perform a relative DRAW
 * @mparam x the x offset.
 * @mparam y the y offset.
 */
#define	draw_lineto(x,y) draw_plot(1,x,y)

/**
 * Draw a circle
 * @mparam x the (canvas) x coordinate of the centre.
 * @mparam y the (canvas) y coordinate of the centre.
 * @mparam z the radius.
 */
#define	draw_circle(x,y,z) \
		do { draw_plot(4, x, y); \
		draw_plot(145, 0, z); } while (0)

/**
 * Draw a filled circle
 * @mparam x the (canvas) x coordinate of the centre.
 * @mparam y the (canvas) y coordinate of the centre.
 * @mparam z the radius.
 */
#define draw_circlefill(x,y,z) \
		do { draw_plot(4, x, y); \
	        draw_plot(153, 0, z); } while (0)

/**
 * Draw a rectangle
 * @mparam x the (canvas) x coordinate.
 * @mparam y the (canvas) y coordinate.
 * @mparam j the width.
 * @mparam k the height.
 */
#define draw_rectangle(x,y,j,k) \
		do { draw_plot(4, x, y); \
		draw_plot(1, j, 0); \
		draw_plot(1, 0, k); \
		draw_plot(1, - (j), 0); \
		draw_plot(1, 0, - (k)); } while(0)

/**
 * Draw a filled rectangle
 * @mparam x the (canvas) x coordinate.
 * @mparam y the (canvas) y coordinate.
 * @mparam j the width.
 * @mparam k the height.
 */
#define draw_rectanglefill(x,y,j,k) \
		do { draw_plot(4, x, y); \
		draw_plot(97, j, k); } while(0)

/**
 * Plot a sprite
 * @param spr_area the sprite area.
 * @param spr_name the sprite name.
 * @param x the (canvas) x coordinate of the left hand side.
 * @param y the (canvas) y coordinate of the bottom.
 * @see sprite_plotspriten()
 * @see sprite_plotspritepn()
 */
void sprite_plotsprite(void *spr_area,
		       char *spr_name,
		       int x,
		       int y);

/**
 * Plot a sprite without canvas translation.
 * @param spr_area the sprite area.
 * @param spr_name the sprite name.
 * @param x the (screen) x coordinate of the left hand side.
 * @param y the (screen) y coordinate of the bottom.
 * @see sprite_plotsprite()
 */
void sprite_plotspriten(void *spr_area,
			char *spr_name,
			int x,
			int y);

/**
 * Plot a scaled sprite without canvas translation.
 * @param spr_area the sprite area.
 * @param spr_name the sprite name.
 * @param x the (screen) x coordinate of the left hand side.
 * @param y the (screen) y coordinate of the bottom.
 * @param xm the x multiplier.
 * @param ym the y multiplier.
 * @param xd the x divider.
 * @param yd the y divider.
 * @param trans the translation table.
 * @see sprite_plotspritept()
 * @see spr_transtab
 */
void sprite_plotspritet(void *spr_area,
			char *spr_name,
			int x,
			int y,
			int xm,
			int ym,
			int xd,
			int yd,
			spr_transtab *trans);

/**
 * Plot a sprite
 * @param spr_area the sprite area.
 * @param spr_ptr the sprite pointer.
 * @param x the (canvas) x coordinate of the left hand side.
 * @param y the (canvas) y coordinate of the bottom.
 * @see sprite_plotsprite()
 * @see sprite_plotspritepn()
 */
void sprite_plotspritep(void *spr_area,
			void *spr_ptr,
			int x,
			int y);

/**
 * Plot a sprite without canvas translation.
 * @param spr_area the sprite area.
 * @param spr_ptr the sprite pointer.
 * @param x the (screen) x coordinate of the left hand side.
 * @param y the (screen) y coordinate of the bottom.
 * @see sprite_plotspritep()
 * @see sprite_plotspriten()
 */
void sprite_plotspritepn(void *spr_area,
			 void *spr_ptr,
			 int x,
			 int y);

/**
 * Plot a scaled sprite without canvas translation.
 * @param spr_area the sprite area.
 * @param spr_ptr the sprite pointer.
 * @param x the (screen) x coordinate of the left hand side.
 * @param y the (screen) y coordinate of the bottom.
 * @param xm the x multiplier.
 * @param ym the y multiplier.
 * @param xd the x divider.
 * @param yd the y divider.
 * @param trans the translation table.
 * @see sprite_plotrawp()
 * @see draw_plotsprite()
 * @see draw_plotsprite56()
 * @see sprite_plotspritet()
 * @see spr_transtab
 */
void sprite_plotspritept(void *spr_area,
			 void *spr_ptr,
			 int x,
			 int y,
			 int xm,
			 int ym,
			 int xd,
			 int yd,
			 spr_transtab *trans);

/**
 * Load a sprite file into memory, allocating memory for it.
 * @param file_name the name of the file.
 * @return the pointer to the sprite area, or NULL if unable to
 * load the file (memory failure, or file not found).
 */
void *sprite_loadsprite(char *file_name);

/**
 * Check to see if the sprite exists.
 * @param spr_area the sprite area.
 * @param spr_name the name of the sprite.
 * @returns zero for sprite does not exist, non-zero for sprite exists.
 * @see sprite_address()
 */
int sprite_spriteexists(void *spr_area,
			char *spr_name);

/**
 * Get the address of a sprite.
 * @param spr_area the sprite area to search.
 * @param spr_name the name of the sprite.
 * @return the pointer to the sprite, or NULL if not found.
 */
void *sprite_address(void *spr_area,
		     char *spr_name);

#define SPRITE_WIMP ((void *)1)

/**
 * The number of colours.
 */
extern unsigned int draw_colours; 			/* Maximum number of colours		 */

/**
 * The current X scale.
 */
extern int draw_scalex;

/**
 * The current Y scale.
 */
extern int draw_scaley;

/**
 * The current X shift (ie. to convert from OS pixels to screen units)
 */
extern int draw_shiftx;

/**
 * The current Y shift (ie. to convert from OS pixels to screen units)
 */
extern int draw_shifty;


/**
 * The maximum X coordinate.
 */
extern int draw_maxx;

/**
 * The maximum Y coordinate.
 */
extern int draw_maxy;

/**
 * The current mode (or mode selector).
 */
extern int draw_mode;

/**
 * The current log base2 of bits per pixel.
 */
extern int draw_bpp;

/**
 * Structure of a draw capline
 * @field join the join type.
 * @field lead_cap the lead cap type.
 * @field tail_cap the tail cap type.
 * @field reserved do not use; reserved.
 * @field mitre_limit the limit of the mitre length for mitre joins.
 * @field lead_capwidth the width of the lead cap.
 * @field lead_caplength the length of the lead cap.
 * @field tail_capwidth the width of the tail cap.
 * @field tail_caplength the length of the tail cap.
 */
typedef struct {
  char join;
  char lead_cap;
  char tail_cap;
  char reserved;
  int mitre_limit;
  short int lead_capwidth;
  short int lead_caplength;
  short int tail_capwidth;
  short int tail_caplength;
} draw_capline;

/**
 * Structure of a draw dash line
 * @field start_distance the distance from the start of the line to the start of the first dash.
 * @field entries the number of items in the dash list.
 * @field elements the dashed elements.
 */
typedef struct {
  int start_distance;
  int entries;
  int elements[1];
} draw_dash;

/**
 * Draw a filled segment.
 * @param path the input path.
 * @param style the fill style.
 * @param trans the transformation matrix (or NULL for identity).
 * @param flat the flatness.
 */
void draw_fill(void *path,
	       int style,
	       int *trans,
	       int flat);

/**
 * Draw a stroked segment.
 * @param path the input path.
 * @param style the fill style.
 * @param trans the transformation matrix (or NULL for identity).
 * @param flat the flatness.
 * @param thick the thickness.
 * @param cap the line cap style.
 * @param dash the line dash style.
 * @see draw_capline
 * @see draw_dash
 */
void draw_stroke(void *path,
		 int style,
		 int *trans,
		 int flat,
		 int thick,
		 draw_capline *cap,
		 draw_dash *dash);

/**
 * Set the current graphics colour.
 * @param col the colour to use (0xBBGGRR00)
 */
void draw_setcolour(int col);

/**
 * Set the font colour.
 * @param handle the font handle.
 * @param back the background colour.
 * @param fore the foreground colour.
 * @note Unlike font_setcolour(), this uses ColourTrans to set the colours correctly.
 * @see font.h
 */
void draw_setfontcolour(int handle,
			int back,
			int fore);

/**
 * Find the font handle
 * @param name the font name.
 * @param w the width in points.
 * @param h the height in points.
 * @return the font handle.
 * @note This is identical to font_find().
 */
int draw_findfont(char *name,
		  int w,
		  int h);

/**
 * Lose a font handle.
 * @param handle the handle to lose.
 * @note This is identical to font_lose();
 */
void draw_losefont(int handle);

/**
 * Draw with the given font/coordinates.
 * @param handle the handle.
 * @param str the string to plot.
 * @param x the (screen) x coordinate.
 * @param y the (screen) y coordinate.
 * @note If you want to draw on a canvas, add __draw_localx and __draw_localy to the coordinates.
 */
void draw_paintfont(int handle,
		    char *str,
		    int x,
		    int y);

/**
 * Draw with the given font/coordinates.
 * @param handle the handle.
 * @param str the string to plot.
 * @param x the (millipoint) x coordinate.
 * @param y the (millipoint) y coordinate.
 * @param trans the pointer to the transformation matrix
 * @param flags the icon flags.
 * @note If you want to draw on a canvas, add __draw_localx and __draw_localy to the coordinates.
 */
void draw_paintfontx(int handle,
		     char *str,
		     int x,
		     int y,
		     int *trans,
		     int flags);

/**
 * Sets the colour to the WIMP colour.
 * @param col bits 0 to 3 are the WIMP colour (0-15) to use, bits 4 to 7 are the GCOL action.
 */
void draw_gcol(int col);

/**
 * Get the information on a sprite pointer.
 * @param spr_ptr the pointer to the sprite.
 * @param xpix the pointer to the variable to hold the width in pixels.
 * @param ypix the pointer to the variable to hold the height in pixels.
 * @param mode the pointer to the variable to hold the sprite mode.
 */
void draw_getspriteinfo(void *spr_ptr,
			int *xpix,
			int *ypix,
			int *mode);

/**
 * Get the information on a namedsprite.
 * @param spr_area the pointer to the sprite area.
 * @param spr_name the pointer to the sprite name.
 * @param xpix the pointer to the variable to hold the width in pixels.
 * @param ypix the pointer to the variable to hold the height in pixels.
 * @param mode the pointer to the variable to hold the sprite mode.
 * @param mask the pointer to the variable to hold the sprite mask state.
 */
void draw_getnamedspriteinfo(void *spr_area,
			     char *spr_name,
			     int *xpix,
			     int *ypix,
			     int *mode,
			     int *mask);

/**
 * Get the mode variable.
 * @param mode the mode to use (-1 for current).
 * @param var_num the variable number to get.
 * @return the value of the mode variable.
 */
int draw_getmodevar(int mode,
		    int var_num);

/**
 * Get the pointed sprite's translation tables.
 * @param spr_ptr the pointer to the sprite.
 * @param sfactor the pointer to the scaling factor buffer.
 * @param ctrans the pointer to the colour translation table buffer.
 */
void sprite_getscalingp(void *spr_ptr,
			int *sfactor,
			char *ctrans);

/**
 * Plot a sprite with scaling and colour translation.
 * @param spr_ptr the sprite pointer.
 * @param x the (screen) x coordinate.
 * @param y the (screen) y coordinate.
 * @param sfactor the pointer to the scaling factor buffer.
 * @param ctrans the pointer to the colour translation table buffer.
 * @see draw_plotsprite()
 * @see draw_plotsprite56()
 */
void sprite_plotrawp(void *spr_ptr,
		     int x,
		     int y,
		     int *sfactor,
		     char *ctrans);

/** 
 * Set the origin.
 * @param x the (screen) x coordinate.
 * @param y the (screen) y coordinate.
 */
void draw_origin(int x,
		 int y);

/**
 * Set the ECF origin.
 * @param x the (screen) x coordinate.
 * @param y the (screen) y coordinate.
 */
void draw_ecforigin(int x,
		    int y);

/**
 * Plot an icon.
 * @param x0 the left hand coordinate.
 * @param y0 the bottom coordinate.
 * @param x1 the right hand coordinate.
 * @param y1 the top coordinate.
 * @param flags the icon flags.
 * @param data the pointer to the icon data buffer.
 * @see icons.h
 * @note The coordinates are either screen or canvas, depending on context - if the icon is being plotted
 * while in the middle of a canvas redraw or update, then they are canvas coordinates. Otherwise, they are
 * screen coordinates. Note that screen coordinates are only recognised by certain OS versions.
 */
void draw_ploticon(int x0,
		   int y0,
		   int x1,
		   int y1,
		   int flags,
		   icon_data *data);

/**
 * Set the screen to a numerical mode.
 * @param mode the mode number.
 * @note This is deprecated, as it does not support mode strings.
 * @note Changing modes should not be done anyway.
 */
void draw_setmode(int mode);

/**
 * Redraw the whole screen.
 */
void draw_redrawscreen(void);

/**
 * Sets the screen mode to 0 (very low bandwidth).
 */
void draw_mode0(void);

/**
 * Plot a sprite.
 * @param spr_ptr the pointer to the sprite.
 * @param style the plot style.
 * @param x the (screen) x coordinate.
 * @param y the (screen) y coordinate.
 * @param sfactor the scaling factor buffer.
 * @param ctrans the colour translation buffer.
 * @see draw_plotsprite56()
 */
void draw_plotsprite(void *spr_ptr,
		     int style,
		     int x,
		     int y,
		     int *sfactor,
		     char *ctrans);

/**
 * Plot a sprite using transformation matrix.
 * @param spr_ptr the pointer to the sprite.
 * @param style the plot style.
 * @param strans the scaling transformation matrix buffer.
 * @param ctrans the colour translation buffer.
 * @see draw_plotsprite()
 */
void draw_plotsprite56(void *spr_ptr,
		       int style,
		       int *strans,
		       char *ctrans);

/**
 * Draw a scaled character on the screen.
 * @param chr the character to plot.
 * @param x the (screen) x coordinate.
 * @param y the (screen) y coordinate.
 * @param sfactor the scaling factors buffer.
 */
void draw_charscaled(char chr,
		     int x,
		     int y,
		     int *sfactor);

/**
 * Set the screen character size.
 * @param w the width
 * @param h the height.
 */
void draw_setcharsize(int w,
		      int h);

/**
 * Raw print character.
 * @param chr the character to print.
 */
void draw_pchar(char chr);

/**
 * @section WIMP colours
 */
#define COLOUR_WHITE      (0)
#define COLOUR_GREY1	  (1)
#define COLOUR_GREY2	  (2)
#define COLOUR_GREY3	  (3)
#define COLOUR_GREY4	  (4)
#define COLOUR_GREY5	  (5)
#define COLOUR_GREY6	  (6)
#define COLOUR_BLACK	  (7)
#define COLOUR_DARKBLUE	  (8)
#define COLOUR_YELLOW	  (9)
#define COLOUR_LIGHTGREEN (10)
#define COLOUR_RED	  (11)
#define COLOUR_CREAM	  (12)
#define COLOUR_DARKGREEN  (13)
#define COLOUR_ORANGE	  (14)
#define COLOUR_LIGHTBLUE  (15)


#ifdef __cplusplus
}
#endif

#endif
